#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/04/16
#Last Modify: 2018/04/16
# deeper learning insights machine learning page 172

import pandas as pd
# Load the data
df = pd.read_csv('/home/egg/projects/deeper_insights_machine_learning/wdbc.data',
header=None)
# Next, we assign the 30 features to a NumPy array X. Using LabelEncoder, we
# transform the class labels from their original string representation (M and
# B) into integers:
from sklearn.preprocessing import LabelEncoder
X = df.loc[:, 2:].values
y = df.loc[:, 1].values
le = LabelEncoder()
y = le.fit_transform(y)
print le.transform(['M', 'B'])

# Split
from sklearn.cross_validation import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, 
                                                    test_size=0.20,
                                                    random_state=1)
# Create the pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.decomposition import PCA
from sklearn.linear_model import LogisticRegression
from sklearn.pipeline import Pipeline
# The Pipeline object takes a list of tuples as input, where the first value in
# each tuple is an arbitrary identifier string that we can use to access the
# individual elements in the pipeline, as we will see later in this chapter,
# and the second element in every tuple is a scikit-learn transformer or
# estimator.
pipe_lr = Pipeline([('scl', StandardScaler()),
                    ('pca', PCA(n_components=2)),
                    ('clf', LogisticRegression(random_state=1))])
pipe_lr.fit(X_train, y_train)
print('Test Accuracy: %.3f' % pipe_lr.score(X_test, y_test))
# Test Accuracy: 0.947
 
# Evaluate k folds
import numpy as np
from sklearn.cross_validation import StratifiedKFold
kfold = StratifiedKFold(y=y_train,
                        n_folds=10,
                        random_state=1)
scores = []
for k, (train, test) in enumerate(kfold):
  pipe_lr.fit(X_train[train], y_train[train])
  score = pipe_lr.score(X_train[test], y_train[test])
  scores.append(score)
  print('Fold: %s, Class dist.: %s, Acc: %.3f' % (k+1,
                                                  np.bincount(y_train[train]),
                                                  score))
#Fold: 1, Class dist.: [256 153], Acc: 0.891
#Fold: 2, Class dist.: [256 153], Acc: 0.978
#Fold: 3, Class dist.: [256 153], Acc: 0.978
#Fold: 4, Class dist.: [256 153], Acc: 0.913
#Fold: 5, Class dist.: [256 153], Acc: 0.935
#Fold: 6, Class dist.: [257 153], Acc: 0.978
#Fold: 7, Class dist.: [257 153], Acc: 0.933
#Fold: 8, Class dist.: [257 153], Acc: 0.956
#Fold: 9, Class dist.: [257 153], Acc: 0.978
#Fold: 10, Class dist.: [257 153], Acc: 0.956
print('CV accuracy: %.3f +/- %.3f' % (np.mean(scores), np.std(scores)))
#CV accuracy: 0.950 +/- 0.029


# Although the previous code example was useful to illustrate how k-fold
# cross-validation works, scikit-learn also implements a k-fold
# cross-validation scorer, which allows us to evaluate our model using
# stratified k-fold cross-validation more efficiently:
from sklearn.cross_validation import cross_val_score
scores = cross_val_score(estimator=pipe_lr,
                         X=X_train,
                         y=y_train,
                         cv=10,
                         n_jobs=1) #how many cores to use, -1=all available
print('CV accuracy scores: %s' % scores)
#CV accuracy scores: [ 0.89130435 0.97826087 0.97826087
#0.91304348 0.93478261 0.97777778
#0.93333333 0.95555556 0.97777778
#0.95555556]
print('CV accuracy: %.3f +/- %.3f' % (np.mean(scores), np.std(scores)))
#CV accuracy: 0.950 +/- 0.029

